@CHARSET "UTF-8";

.masthead { 
	background:rgb(235, 250, 229);
	border:solid rgb(128, 141, 119) 0px;
	border-bottom-width: 1px;
}
.masthead .mastheadLeft ul, .masthead .mastheadRight ul {
	border:solid rgb(128, 141, 119) 0px;
	color: rgb(128, 141, 119);
}
.masthead .mastheadLeft ul li a, .masthead .mastheadRight ul li a {
	color: rgb(128, 141, 119);
}
.masthead #phedex a{
	color:white;
	background:rgb(128, 141, 119);
}
/* This next bit is for site dependant masthead */
.mastheadfooter {
	background:rgb(128, 141, 119);
}
.mastheadfooter ul {
	color:white;
}
.mastheadfooter ul li {
	color:white;	
}
.mastheadfooter ul li a, .mastheadfooter ul li a:visited{
	color:white;
}
.mastheadfooter ul li a:hover{
	color:white;
}
.mastheadfooter ul li a:active{
	color:white;
}