"""
_JobTypeCountByState_

Oracle implementation of Monitoring.JobTypeCountByState
"""
from __future__ import print_function, division

from WMCore.WMBS.MySQL.Monitoring.JobTypeCountByState import JobTypeCountByState as MySQLJobTypeCountByState

class JobTypeCountByState(MySQLJobTypeCountByState):
    pass
