#!/usr/bin/env python
"""
_SetLocationByLFN_

Oracle implementation of Files.SetLocationByLFN
"""




from WMCore.WMBS.MySQL.Files.SetLocationByLFN import SetLocationByLFN as MySQLSetLocationByLFN

class SetLocationByLFN(MySQLSetLocationByLFN):
    pass
