#!/usr/bin/env python
"""
_LoadFilesByBlock_

Oracle implementation of DBS3Buffer.LoadFilesByBlock
"""

from WMComponent.DBS3Buffer.MySQL.LoadFilesByBlock import LoadFilesByBlock as MySQLLoadFilesByBlock

class LoadFilesByBlock(MySQLLoadFilesByBlock):
    pass
