# -*- coding: utf-8 -*-
"""
_GetRunningSlots_

Oracle implementation of Locations.GetRunningSlots
Created on Mon Jun 18 13:49:39 2012

@author: dballest
"""

from WMCore.WMBS.MySQL.Locations.GetRunningSlots import GetRunningSlots as MySQLGetRunningSlots

class GetRunningSlots(MySQLGetRunningSlots):
    """
    Same as MySQL version

    """
    pass
