#!/usr/bin/env python
"""
_ListSubTypes_

Oracle implementation of Monitoring.ListSubTypes
"""




from WMCore.WMBS.MySQL.Monitoring.ListSubTypes import ListSubTypes \
    as ListSubTypesMySQL

class ListSubTypes(ListSubTypesMySQL):
    pass
