#!/usr/bin/env python
"""
_GetParentIDsByID_

Oracle implementation of GetParentIDsByID
"""




from WMCore.WMBS.MySQL.Files.GetParentIDsByID import GetParentIDsByID \
     as GetParentIDsMySQL

class GetParentIDsByID(GetParentIDsMySQL):
    pass
